@@ -1,4 +1,5 @@
-from .files import reader, tokens, nopen, header, is_newer_b
+from .files import (reader, tokens, nopen, header, is_newer_b,
+    basestring, int_types)
 import sys
 from .pool import pool, pmap
 
@@ -33,6 +34,18 @@ def groupby(iterable, key=0, filter=None):
     for label, grp in igroupby(iterable, key):
         yield label, list(grp)
 
+try:
+    from itertools import izip_longest
+except ImportError:
+    from itertools import zip_longest as izip_longest
+
+def groups_of(n, iterable):
+    """
+    >>> groups_of(2, range(5))
+    """
+    args = [iter(iterable)] * n
+    for x in izip_longest(*args):
+        yield [v for v in x if v is not None]
 
 def main():
     #import argparse
