@@ -21,6 +21,15 @@ import bz2
 from subprocess import Popen, PIPE
 import csv
 
+if sys.version_info.major < 3:
+    int_types = (int, long)
+    urlopen = urllib.urlopen
+    basestring = basestring
+else:
+    int_types = (int,)
+    basestring = str
+    urlopen = urllib.request.urlopen
+
 dialect = csv.excel
 
 class ProcessException(Exception): pass
@@ -40,13 +49,14 @@ def process_iter(proc, cmd=""):
         else:
             proc.wait()
             if proc.returncode not in (0, None):
-                print >>sys.stderr, "cmd was:", cmd
+                sys.stderr.write("cmd was:%s\n" % cmd)
                 raise ProcessException(proc.stderr.read())
             s = proc.stderr.read().strip()
             if len(s) != 0:
-                print >>sys.stderr, s
+                sys.stderr.write(str(s))
+                sys.stderr.write('\n')
 
-def nopen(f, mode="rb"):
+def nopen(f, mode="r"):
     r"""
     open a file that's gzipped or return stdin for '-'
     if f is a number, the result of nopen(sys.argv[f]) is returned.
@@ -55,24 +65,24 @@ def nopen(f, mode="rb"):
     (True, True)
 
     >>> nopen(sys.argv[0])
-    <open file '...', mode 'r...>
+    <...file...>
 
     # expands user and vars ($HOME)
-    >>> nopen("~/.bashrc"), nopen("$HOME/.bashrc")
-    (<open file '...', mode 'r...>, <open file '...', mode 'r...>)
+    >>> nopen("~/.bashrc").name == nopen("$HOME/.bashrc").name
+    True
 
     # an already open file.
     >>> nopen(open(sys.argv[0]))
-    <open file '...', mode 'r...>
+    <...file...>
 
     >>> nopen(0)
-    <open file '...', mode 'r...>
+    <...file...>
 
     Or provide nicer access to Popen.stdout
     >>> files = list(nopen("|ls"))
-    >>> assert 'setup.py\n' in files, files
+    >>> assert 'setup.py\n' in files or b'setup.py\n' in files, files
     """
-    if isinstance(f, (int, long)):
+    if isinstance(f, int_types):
         return nopen(sys.argv[f], mode)
 
     if not isinstance(f, basestring):
@@ -87,14 +97,28 @@ def nopen(f, mode="rb"):
         return p
 
     if f.startswith(("http://", "https://", "ftp://")):
-        fh = urllib.urlopen(f)
+        fh = urlopen(f)
         if f.endswith(".gz"):
             return ungzipper(fh)
-        return fh
+        if sys.version_info.major < 3:
+            return fh
+        import io
+        return io.TextIOWrapper(fh)
     f = op.expanduser(op.expandvars(f))
+    if f.endswith((".gz", ".Z", ".z")):
+        fh = gzip.open(f, mode)
+        if sys.version_info.major < 3:
+            return fh
+        import io
+        return io.TextIOWrapper(fh)
+    elif f.endswith((".bz", ".bz2", ".bzip2")):
+        fh = bz2.BZ2File(f, mode)
+        if sys.version_info.major < 3:
+            return fh
+        import io
+        return io.TextIOWrapper(fh)
+
     return {"r": sys.stdin, "w": sys.stdout}[mode[0]] if f == "-" \
-         else gzip.open(f, mode) if f.endswith((".gz", ".Z", ".z")) \
-         else bz2.BZ2File(f, mode) if f.endswith((".bz", ".bz2", ".bzip2")) \
          else open(f, mode)
 
 def ungzipper(fh, blocksize=16384):
@@ -127,7 +151,7 @@ def header(fname, sep="\t"):
     just grab the header from a given file
     """
     fh = iter(nopen(fname))
-    h = tokens(fh.next(), sep)
+    h = tokens(next(fh), sep)
     h[0] = h[0].lstrip("#")
     return h
 
@@ -149,16 +173,23 @@ def reader(fname, header=True, sep="\t", skip_while=None):
 
     skip_while = lambda toks: toks[0].startswith('#')
 
-    >>> from StringIO import StringIO
+    >>> import sys
+    >>> if sys.version_info.major < 3:
+    ...     from StringIO import StringIO
+    ... else:
+    ...     from io import StringIO
     >>> get_str = lambda : StringIO("a\tb\tname\n1\t2\tfred\n11\t22\tjane")
-    >>> list(reader(get_str()))
-    [{'a': '1', 'b': '2', 'name': 'fred'},
-     {'a': '11', 'b': '22', 'name': 'jane'}]
+    >>> expected = [{'a': '1', 'b': '2', 'name': 'fred'},
+    ... {'a': '11', 'b': '22', 'name': 'jane'}]
+    >>> list(reader(get_str())) == expected
+    True
 
-    >>> list(reader(get_str(), header=False))
-    [['a', 'b', 'name'], ['1', '2', 'fred'], ['11', '22', 'jane']]
+    >>> expected = [['a', 'b', 'name'], 
+    ...             ['1', '2', 'fred'], ['11', '22', 'jane']]
+    >>> list(reader(get_str(), header=False)) == expected
+    True
     """
-    if isinstance(fname, (int, long)):
+    if isinstance(fname, int_types):
         fname = sys.argv[fname]
     if not isinstance(fname, basestring) and \
         isinstance(fname, types.GeneratorType):
@@ -186,9 +217,9 @@ def reader(fname, header=True, sep="\t", skip_while=None):
 
     if skip_while:
         from itertools import chain
-        l = line_gen.next()
+        l = next(line_gen)
         while skip_while(l):
-            l = line_gen.next()
+            l = next(line_gen)
         line_gen = chain.from_iterable(([l], line_gen))
 
     # they sent in a class or function that accepts the toks.
@@ -205,7 +236,7 @@ def reader(fname, header=True, sep="\t", skip_while=None):
         header = True
 
     if header == True:
-        header = line_gen.next()
+        header = next(line_gen)
         header[0] = header[0].lstrip("#")
 
     if header:
